///////////////////////////////////////////////////////////////////////////////
//	FILENAME:	eTImageComponent.h
//	SUMMARY:	Interface for a container of imageable data.
//	SUPERCLASS:	Object
//	INTERFACE:	None, but it "exports" eTImageComponentIcon.
//	PROTOCOLS:	<ComponentData,
//				ETFDSupport,HTMDSupport,LaTeXSupport>
//	AUTHOR:		Rohit Khare
//	COPYRIGHT:	(c) 1994 California Institure of Technology, eText Project
///////////////////////////////////////////////////////////////////////////////
//	DESCRIPTION
//		This is a subclass that externalizes image components. Manages
//	an NXImage object, eps to tiff, tiff to eps and tiff to gif conversion
///////////////////////////////////////////////////////////////////////////////
//	HISTORY
//	01/22/95:	Added call to handle caption strings.
//	07/19/94:	Rearchitected; see eTComponent and Actors/eTComponent.rtf.
//	07/14/94:	Created. Aliases the NeXTSTEP NXImage API.
///////////////////////////////////////////////////////////////////////////////

#import "eTComponent.h"

@interface eTImageComponent:eTComponent <ComponentData,ETFDSupport,HTMDSupport,LaTeXSupport>
{
	NXImage	*theImage;
	BOOL	isShared;
}

+ newImageNamed:(const char *)theName;
- theImage;
- (BOOL)isShared; // this is valid, but not suggested
// semi-private API
- useImage:(NXImage *)newImage 
	  name:(const char *) newComponentName
	  path:(const char *) newCurrentPath
	shared:(BOOL) newShared;
// VERY PRIVATE API
- writeHTML:(NXStream *)stream forView:view withCaption:(NXAtom)caption;
- setImage:newImg;
- setShared:(BOOL)newState;
- registerError;
@end